<?php

  /**
   * @file
   *  Coding standard in this file. 
   *  Public function:   All functions start with Capital Letter. these functions needs to be used
   *            in other files under the same module, or needs to be exposed to outside the
   *            module by .module file.
   *  Private function:  All functions start with "_", this functions is limited in this file.
   *  Database Table  :   All table names need to be defined as static variable defined by using php "define"
   */

   /**
    * This file contains library functions used by all of the other php files under the same module.
    * if one function is used at several files, this functions should be placed in this file.
    */
  
  define("ADMIN", "admin");
    
  define("GROUP_ADMIN", "group admin");
  
  define("ORGANIZATION_ADMIN", "organization admin");
    
  define("GROUP_STATISTIC_EXCEL", 'group_statistic_excel');
    
  //base directory which contains all of the sco contents.
  define("MODULE_NAME", 'simple_LMS');
  
  define("sco_url_basedir", '/webserver/www/html/sites/all/scos/');
  
  define("sco_reserved_path", '/sites/all/scos/');
  
  define("sco_api_file", '/api.php');
  
  define("SCO_TABLE", 'simple_LMS_SCO');
  
  define("SCOLearner_TABLE", 'simple_LMS_SCOLearner');
  
  define("SL_Interaction_TABLE", 'simple_LMS_SCOLearner_Interaction');
  
  define("INVALIDATE_DATE", "0000-00-00 00:00:00");
  
  define("CMI_INITIAL_TIME", "0000:00:00");
  
  define("CMI_CREDIT", "credit");
  
  /**
   * According to the SCORM specificiation, cmi.core.lesson_status should be initialized to
   * not attempted, the final status according to the follow: 
   */
  define("CMI_INITIAL_LESSON_STATUS", "not attempted");
  
  /**
   * According to the SCORM specification, cmi.core.entry should be initialized to ab initio 
   * when a course first starts. Then, when LMSFinish() is called, the LMS should follow a very 
   * simple rule:
   * (1): If cmi.core.exit has already been set to 'suspend' when the course exits, 
   *    then cmi.core.entry should be set to 'resume'.
   * (2): In all other cases, cmi.core.entry should be set to '' (an empty string).
   */
  define("CMI_INITIAL_ENTRY", "ab initio");
  
  define("CMI_SUSPEND_ENTRY", "resume");
  
  define("CMI_OTHER_ENTRY", "");
  
  /**
   * get one or more records from the simple_LMS_SCOLearner table
   * based on the known user id and sco id.
   */
  function simple_LMS_Get_item_from_scolearner($uid, $scoid, $sid, $retobject = FALSE) {
    $sql = "SELECT * FROM {". SCOLearner_TABLE . "} WHERE scoid = %d AND uid = %d AND sid = %d";
  
    $result = db_query($sql, $scoid, $uid, $sid);
    
    if ($result && count($result) == 1 ) {
      $r = NULL;
      if (!$retobject) {
		$r = db_fetch_array($result);
      } 
      else {
        $r = db_fetch_object($result);
      }
      return $r;
    }
    
    return NULL;
  }
  
  function simple_LMS_Update_scolearner(&$scolearner_item) {
    return drupal_write_record( SCOLearner_TABLE, $scolearner_item, array('sid') );
  }
  
  /**
   * Variable name convert table
   * --- scorm standard name  ------------- name in database ------
   * |-- cmi.core.lesson_location --|----- lesson_location -------|
   * |-- cmi.core.credit -----------|------ credit ---------------|
   * |-- cmi.core.lesson_status ----|------ lesson_status --------|
   * |-- cmi.core.entry ------------|------ entry ----------------|
   * |-- cmi.core.total_time -------|------ total_time -----------|
   * |-- cmi.core.session_time -----|------ session_time ---------|
   * |-- cmi.core.score.raw --------|------ score ----------------|
   * |-- cmi.core.suspend_data -----|------ suspend_data ---------|
   * |-- cmi.core.exit -------------|------ exit -----------------|
   * --------------------------------------------------------------
   * All name end with _SD means that they represent standard name
   * All name end with _DB means that they are corresponding name in database
   */
  define("CMI_CORE_LESSON_LOCATION_SD",  "cmi.core.lesson_location");
  define("CMI_CORE_CREDIT_SD",      "cmi.core.credit");
  define("CMI_CORE_LESSON_STATUS_SD",   "cmi.core.lesson_status");
  define("CMI_CORE_ENTRY_SD",       "cmi.core.entry");
  define("CMI_CORE_TOTAL_TIME_SD",    "cmi.core.total_time");
  define("CMI_CORE_SESSION_TIME_SD",    "cmi.core.session_time");
  define("CMI_CORE_SCORE_RAW_SD",     "cmi.core.score.raw");
  define("CMI_SUSPEND_DATA_SD",       "cmi.suspend_data");
  define("CMI_CORE_EXIT_SD",        "cmi.core.exit");

  define("CMI_CORE_LESSON_LOCATION_DB",  "lesson_location");
  define("CMI_CORE_CREDIT_DB",      "credit");
  define("CMI_CORE_LESSON_STATUS_DB",   "lesson_status");
  define("CMI_CORE_ENTRY_DB",       "entry");
  define("CMI_CORE_TOTAL_TIME_DB",    "total_time");
  define("CMI_CORE_SESSION_TIME_DB",    "session_time");
  define("CMI_CORE_SCORE_RAW_DB",     "score");
  define("CMI_SUSPEND_DATA_DB",       "suspend_data");
  define("CMI_CORE_EXIT_DB",        "exit_way");
  
  function simple_LMS_Mappingname($var) {
    $var = trim($var);
  
    switch ($var) {
      case CMI_CORE_LESSON_LOCATION_SD:   return CMI_CORE_LESSON_LOCATION_DB;
      
      case CMI_CORE_CREDIT_SD:       return CMI_CORE_CREDIT_DB;
        
      case CMI_CORE_LESSON_STATUS_SD:    return CMI_CORE_LESSON_STATUS_DB;
        
      case CMI_CORE_ENTRY_SD:        return CMI_CORE_ENTRY_DB;
        
      case CMI_CORE_TOTAL_TIME_SD:    return CMI_CORE_TOTAL_TIME_DB;
      
      case CMI_CORE_SESSION_TIME_SD:    return CMI_CORE_SESSION_TIME_DB;
        
      case CMI_CORE_SCORE_RAW_SD:      return CMI_CORE_SCORE_RAW_DB;
        
      case CMI_SUSPEND_DATA_SD:      return CMI_SUSPEND_DATA_DB;
        
      case CMI_CORE_EXIT_SD:        return CMI_CORE_EXIT_DB;
      
      default:  return NULL;
    }
  }
  
  function simple_LMS_Mappingname_Humanreadable($db_var) {
    switch ($db_var) {  
      case CMI_CORE_LESSON_STATUS_DB:    return "Lesson Status";
      
      case CMI_CORE_TOTAL_TIME_DB:    return "Total Time";
      
      case CMI_CORE_SESSION_TIME_DB:    return "Last Session Time";
      
      case CMI_CORE_SCORE_RAW_DB:      return "Score";
      
      default:
        return NULL;
    }
  }
  
  /**
   * This function return an array that contains all of the sco contents a user has
   * parameter: user id
   * return:    array of array which contains "sco id", "sco name", "sco type", "sco category"  
   */
  function simple_LMS_Get_user_contents($userid) {
  
    $sql_statement = "SELECT b.sid, a.scoid, a.nid, a.sconame, a.scotype, a.scocategory, b.expiration_date FROM {" . SCO_TABLE . "} a, {" . SCOLearner_TABLE . '} b WHERE b.uid=%d AND a.scoid=b.scoid';
  
    $resultary = array();
    
    $result = db_query($sql_statement, $userid);
    
    if ($result) {
    
      while ($sco = db_fetch_object($result)) {
      
        $resultary[$sco->sid] = array(
            'sid'            => $sco->sid,
            'scoid'          => $sco->scoid,
            'product_nid'    => $sco->nid,
            'sconame'        => $sco->sconame,
            'scotype'        => $sco->scotype,
            'scocategory'    => $sco->scocategory,
            'expirationdate' => $sco->expiration_date,
        );
      }
    }
    
    return $resultary;
  }
  
  /**
   * Get the expiration span of each scorm product.
   * it needs to turn the revision of product off so that for
   * each nid, there is only one specific corresponding product. 
   */
  function simple_LMS_Get_production_expiration_span($product_nid) {
    $sql = "SELECT * FROM {content_type_product} WHERE nid=%d";
    
    $r = db_query($sql, $product_nid);
    
    if ($r) {
    
      $obj = db_fetch_object($r);
      
      return $obj->field_expiration_span_value;
    }
    
    return NULL;
  }
  
  function simple_LMS_Buy_courseproduct($uid, $nid) {  
    if ( !is_null( $sco = simple_LMS_Get_sco_content_object_by_nid($nid) ) ) {
      return simple_LMS_Add_user_sco_relationship($uid, $sco->scoid, $nid);
    }
  }
  
  /**
   * This function is to initialize the "simple_LMS_SCOLearner" table entry
   * to build the relationship between one user and one course.
   * While, the real relationship between users and scos is many-to-many.
   */
  function simple_LMS_Add_user_sco_relationship($userid, $scoid, $product_nid) {
    //make sure, the record is unique. no longer need this call (QUANTITY FIX)
    //if ( simple_LMS_Get_item_from_scolearner($userid, $scoid, $sid) != NULL ) return;
    
    //variable_set("TEST BUYING AGAIN, PRODUCT_NID", $product_nid);
    
    if ( !is_null($weeks_span = simple_LMS_Get_production_expiration_span($product_nid)) ) {
      $new_record = array();
      
      $new_record['scoid']              = $scoid;
      $new_record['uid']                = $userid;
      $new_record['created_date']          = simple_LMS_get_currenttime();
      $new_record['start_date']            = INVALIDATE_DATE;
      $new_record['finish_date']            = INVALIDATE_DATE;
      $new_record['quiz_time']           = CMI_INITIAL_TIME;
      $new_record['expiration_date']        = simple_LMS_Get_futureTime($weeks_span);
      $new_record['interaction_list']        = NULL;
      $new_record[CMI_CORE_LESSON_LOCATION_DB]  = "Simple LMS";
      $new_record[CMI_CORE_CREDIT_DB]        = CMI_CREDIT;
      $new_record[CMI_CORE_LESSON_STATUS_DB]    = CMI_INITIAL_LESSON_STATUS;
      $new_record[CMI_CORE_ENTRY_DB]        = CMI_INITIAL_ENTRY;
      $new_record[CMI_CORE_TOTAL_TIME_DB]      = CMI_INITIAL_TIME;
      $new_record[CMI_CORE_SESSION_TIME_DB]     = CMI_INITIAL_TIME;
      $new_record[CMI_CORE_EXIT_DB]        = NULL;
      $new_record[CMI_CORE_SCORE_RAW_DB]        = NULL;
      $new_record[CMI_SUSPEND_DATA_SD]      = NULL;
      
      return drupal_write_record(SCOLearner_TABLE, $new_record);
    }
  }
  
  /**
   * This function is used to add a new sco content into the database, 
   * it firstly needs to read the imsmanifest.xml file of sco content to get 
   * the entry point which is a html file, then it need to read the mastery score based on which
   * to decide if this sco content is course or quiz.
   */
  function simple_LMS_Add_new_scocontent($sconame) {
    if ( !is_null( $nid = simple_LMS_Get_course_product($sconame) ) ) {
  
      global $user;
    
      $tmpdir = sco_url_basedir . $sconame . '/';
      
      $attributes = simple_LMS_read_from_imsmanifestXML($tmpdir);
    
      $fileinfo = array();
      
      $fileinfo['nid']      = $nid;
      $fileinfo['sconame']     = $sconame;
      $fileinfo['scotype']    = drupal_strlen($attributes['masteryscore']) > 0 ? 'quiz' : 'course'; 
      $fileinfo['scolinks']    = sco_reserved_path . $sconame . '/' . $attributes['entry_point'];
      $fileinfo['scocategory']  = NULL; //not decided yet.
      $fileinfo['cdata']      = simple_LMS_get_currenttime();
      $fileinfo['cperson']    = $user->uid;
      $fileinfo['masteryscore']  = $attributes['masteryscore'];
    
      drupal_write_record(SCO_TABLE, $fileinfo);
      
    }
  }
  
  function simple_LMS_Get_sco_content_object_by_nid($nid) {
    $sql = "SELECT * FROM {" . SCO_TABLE . "} WHERE nid = %d";
    
    $result = db_query($sql, $nid);
    
    if ($result) { 
      $r = db_fetch_object($result);
      return $r;
    }
    
    return NULL;
  }
  
  function simple_LMS_Get_sco_contents_object($scoid) {
    $sql = "SELECT * FROM {" . SCO_TABLE . "} WHERE scoid = %d";
    
    $result = db_query($sql, $scoid);
    
    if ($result) {
      
      $r = db_fetch_object($result);
      
      return $r;
    }
    
    return NULL;
  }
  
  function simple_LMS_Get_question_answers($sid, $sort_byQID = FALSE) {
    $sql = "SELECT * FROM {" . SL_Interaction_TABLE . "} WHERE sid = %d";

    $result = db_query($sql, $sid);

    if ($result) {
      $objary = array();

      while ( $r = db_fetch_object($result) ) {
          $objary[$r->id] = $r;
      }

      if ($sort_byQID) {
        $objary = simple_LMS_sort_question_answers_byQID($objary);
      }

      return $objary;
    }
  }

  function simple_LMS_question_compare($a , $b) {
    $str_1 = $a->id;

    $str_2 = $b->id;

    $id1 = floatval( str_replace("_", ".", str_replace("Question", "0", $str_1)) );

    $id2 = floatval( str_replace("_", ".", str_replace("Question", "0", $str_2)) );

    if ($id1 >= $id2) {
      return 1;
    }
    else {    
      return -1;
    }
  }

  /**
   * Sort question answers by using question ID
   * currently, question id is Question_
   */
  function simple_LMS_sort_question_answers_byQID($objary) {

    usort($objary, "simple_LMS_question_compare");
    
    return $objary;
  }

  /**
   * Has this course expired?
   */
  function simple_LMS_Is_expiration_date_past($expiration_time) {
    if ( strtotime($expiration_time) > time() ) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Get future time in format "year-month-day hour:minute:second" 
   */
  function simple_LMS_Get_futureTime($weeks_later) {  
    $time = "+". $weeks_later . "week";
    
    return date('Y-m-d H:i:s', strtotime($time) );
  }

  /**
   * Get current time in format "year-month-day hour:minute:second" 
   */
  function simple_LMS_get_currenttime() {
    return date('Y-m-d H:i:s');
  }

  /**
   * Currently, this function only support one content in a SCO package. 
   * return values;
   * returnary['entry_point']   :   html links to the content
   * returnary['masteryscore']  :   pass score, currently I used this attribute to see 
   *                  if the content is course or quiz.
   *                  
   */
  function simple_LMS_read_from_imsmanifestXML($dirname) {
    $returnary = array();

    $manifestfile = $dirname . "imsmanifest.xml";
    
    $xmlfile = new DomDocument;
    
    $xmlfile->preserveWhiteSpace = FALSE;

    $xmlfile->load($manifestfile);

    $manifest = $xmlfile->getElementsByTagName('manifest');

    $adlcp = $manifest->item(0)->getAttribute('xmlns:adlcp');

    //resource part of imsmanifest.xml
    $resourceslist = $xmlfile->getElementsByTagName('resources');

    foreach ($resourceslist as $resourceslistrow) {

      $resourcelist = $resourceslistrow->getElementsByTagName('resource');

      foreach ($resourcelist as $resourcelistrow) {
        $returnary['entry_point'] = $resourcelistrow->getAttribute('href');
        break;
      }

      break;
    }

    //organizations part of the imsmanifest.xml
    $organizationslist = $xmlfile->getElementsByTagName('organizations');

    foreach ($organizationslist as $organizationslistrow) {

      $organizationlist = $organizationslistrow->getElementsByTagName('organization');

      foreach ($organizationlist as $organizationlistrow) {

        $itemslist = $organizationlistrow->getElementsByTagName('item');

        foreach ($itemslist as $itemslistrow) {
          $masteryscoretag = $itemslistrow->getElementsByTagNameNS($adlcp, 'masteryscore');
          $returnary['masteryscore'] = $masteryscoretag->item(0)->nodeValue;
          break;
        }

        break;
      }

      break;
    }

    return $returnary;
  }

  /**
   * check if the varname is one of the cmi.interaction.n... variables
   * if it is, it is added into the array.
   */
  function simple_LMS_Filter_cmi_interactions($varname, $varvalue, &$interary) {    
    $fixedstr = "cmi.interactions.";

    //notice, here I need to use !== instead of !=
    if ( strpos($varname, $fixedstr) !== FALSE) {
      $pos = strpos($varname, ".", drupal_strlen($fixedstr) );

      $main_key = drupal_substr($varname, 0, $pos);

      $sub_key = drupal_substr($varname, $pos + 1, (drupal_strlen($varname) - $pos - 1) );

      $interary[$main_key][$sub_key] = $varvalue;
    }
  }

  /**
   * Update simple_LMS_SCOLearner_Interaction table.
   *
   * Business logic (currently, I am using choice 1):
   * choice 1: need to delete all of the previous record first then add the new record.
   * choice 2: if the record exists in table, no more new record can be written into the table again.
   * 
   */
  function simple_LMS_Update_scolearner_interaction($interary, $sid) {
    if ( count($interary) > 0 ) {
      //$r = db_query("DELETE FROM {" . SL_Interaction_TABLE . "} WHERE sid = %d", $sid );

      //if($r != FALSE) {
        foreach ($interary as $subary) {
          $subary['sid'] = $sid;

          $subary['time'] = date("Y-m-d ") . $subary['time'];

          $t = db_query("DELETE FROM {" . SL_Interaction_TABLE . "} WHERE sid = %d AND id = \"%s\" ", $sid, $subary['id']);

          drupal_write_record(SL_Interaction_TABLE, $subary);
        }
      //}
    }
  }

  /**
   * This function is to find the Ubercart product with
   * the same name as the course name. only if this product
   * exist, this course can be activated.
   */
  function simple_LMS_Get_course_product($sconame) {
    $sql = "SELECT nid FROM {uc_products} WHERE model = '%s'";

    $r = db_query($sql, $sconame);
    
    if ($r && count($r) == 1) {
      $nid = db_fetch_object($r);

      return $nid->nid;
    }

    return NULL;
  }

  /**
   * THIS FUNCTION WAS WRONG!!! /////////// IT HAS BEEN CHANGED
   * This function will be reworked to not rely on the user who is view this data
   * This function is to find all users in the same group 
   * if current user is administrator of the group.
   * This function based on "og" module.
   * and the drupal users table.
   *
   * input (1): uid: user id of administrator
   *     (2): include_self: if the final result include administrator itself.
   *      
   * return array of ( group description + user object array ).
   */
  function simple_LMS_Get_users_in_group($nid, $include_self = FALSE, $pager_count = NULL) {
    $userary = array();
    global $user;

    $result = NULL;

    // if my uid happens to appear then show it regardless 
    //(if you are not in the data already then you wont show up)
    if ( $include_self == TRUE ) {
      // nested sql statement
      $sql_user = "SELECT uid,name,mail FROM {users} WHERE uid IN ( SELECT uid FROM {og_uid} WHERE nid=%d )";

      if ( is_null($pager_count) ) {
        $result = db_query($sql_user, $nid);
      } 
      else {
        $parameters = array($nid);
        
        $result = pager_query($sql_user, $pager_count, 0, NULL, $parameters);
      }
    }
    // dont include my uid in this search
    else {
      // nested sql statement grab user info from grab users in group
      $sql_user = "SELECT uid, name, mail FROM {users} WHERE uid IN ( SELECT uid FROM {og_uid} WHERE nid=%d AND uid<>%d )";

      if ( is_null($pager_count) ) {

        $result = db_query($sql_user, $nid, $user->uid);
      }
      else {
        $parameters = array($nid, $user->uid);
        $result = pager_query($sql_user, $pager_count, 0, NULL, $parameters);
      }
    }

    if ($result != NULL) {
      
      //$userary[$nid] = array($og->og_description);
      while ( $userobj = db_fetch_object($result) ) {
        
        $userary[$nid][$userobj->uid] = $userobj;
      }
    }    
    return $userary;
    return NULL;
  }

   /**
    * get the information of specific user
    */
  function simple_LMS_Get_user_info($uid) {
    $sql = "SELECT * FROM {users} WHERE uid=%d";

    $r = db_query($sql, $uid);

    if ($r) {

      $obj = db_fetch_object($r);

      return $obj;
    }

    return NULL;
  }

  /**
   * Get all questions answers for all members of a group.
   *
   * return: array of array;
   */
  function simple_LMS_Get_question_answers_forgroup($group_id, $scoid) {

      $scolearner_items = simple_LMS_Get_scolearner_items_for_group($group_id, $scoid);

      $question_answers_list = array();

      foreach ($scolearner_items as $uid => $scoleaner_itesm_obj) {
        $answers_list = simple_LMS_Get_question_answers($scoleaner_itesm_obj->sid);

        $question_answers_list[$uid] = $answers_list;

      }
      return $question_answers_list;
  }

  /**
   * Get all questions answers from all of the people who own the course
   */
  function simple_LMS_Get_question_answers_From_allsite($scoid) {
    $sql = "SELECT * FROM {" . SCOLearner_TABLE . "} WHERE scoid=%d";

    $r = db_query($sql, $scoid);

    if ($r) {
        $question_answers_list = array();

        while ( $obj = db_fetch_object($r) ) {
            $answers_list = simple_LMS_Get_question_answers($obj->sid);
            $question_answers_list[$obj->uid] = $answers_list;
        }
        return $question_answers_list;
    }
    return NULL;
  }

   /**
    * Get all scolearner items of the members of a specific group
    */
  function simple_LMS_Get_scolearner_items_for_group($group_id, $scoid) {
    $sql_statement = "SELECT * FROM {" . SCOLearner_TABLE . "} WHERE scoid = %d AND uid IN ( SELECT uid FROM {og_uid} WHERE nid=%d )";

    $r = db_query($sql_statement, $scoid, $group_id);

    if ($r) {

      $objary = array();

      while ( $obj = db_fetch_object($r) ) {
          $objary[$obj->uid] = $obj;///////////// this is going to break our calls if one user has more than one (QUANTITY FIX)
      }

      return $objary;
    }

    return NULL;
  }

  /**
   * Get all sco contents the members of a group have bought.
   * 
   * return array of objects
   */
  function simple_LMS_Get_sco_contents_of_group($group_id) {
    $sql_statement = "SELECT * FROM {" . SCO_TABLE . "} WHERE scoid IN ( SELECT scoid FROM {" . SCOLearner_TABLE ."} WHERE uid IN ( SELECT uid FROM {og_uid} WHERE nid=%d ) )";
    $r = db_query($sql_statement, $group_id);

    if ($r) {
      $objary = array();

      while ( $obj = db_fetch_object($r) ) {
        $objary[$obj->scoid] = $obj;
      }

      return $objary;
    }

    return NULL;
  }

   /**
    * Check if the user is the administrator of a specific group
    */
  function simple_LMS_Is_admin_of_group($uid, $group_id) {
    $sql_statement = "SELECT * FROM {og_uid} WHERE uid=%d AND is_admin=1";

    $r = db_query($sql_statement);

    if ($r && count($r) > 0) {
      return TRUE;  
    }

    return FALSE;
  }

  /**
   * check if the user is the admin of the web site
   * because user can have multiple roles, we need to do the iteration.
   * input: user object
   */
  function simple_LMS_Is_admin_of_website_byUserObj($user) {
    foreach ($user->roles as $value) {
      if ($value == ADMIN) return TRUE;
    }
    return FALSE;
  }

   /**
    * check if the user is a group admin.
    * because user can have multiple roles, we need to do the iteration.
    * input: user object
    */
  function simple_LMS_Is_admin_by_role($user, $og) {
    foreach ($user->roles as $value) {
      if ($value == $og) return TRUE;
    }
    return FALSE;
  }

  /**
   * check if the user is the admin of the web site
   * because user can have multiple roles, we need to do the iteration.
   * input: user id
   */
  function simple_LMS_Is_admin_of_website_byUserId($uid) {
    $sql_1 = "SELECT rid FROM {users_roles} WHERE uid = %d";
    
    $r_1 = db_query($sql_1, $uid);
    
    if ($r_1) {
    
      while ( $obj = db_fetch_object($r_1) ) {
        $sql_2 = "SELECT name FROM {role} WHERE rid=%d";
        
        $r_2 = db_query($sql_2, $obj->rid);
        
        if ($r_2) {
        
          $o = db_fetch_object($r_2);
          
          if ($o->name == ADMIN) return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
   * Convert Scorm time format 0000:00:00 to the integer which represents the seconds.
   */
  function simple_LMS_calculate_seconds($scorm_time) {
    $time = explode(':', $scorm_time);
    
    $totalseconds = intval($time[0]) * 60 * 60 + intval($time[1]) * 60 + intval($time[2]);
    
    return $totalseconds;
  }
  
  function simple_LMS_convert_second2hour_format($second) {
    $hour = (float)($second/3600);
    
    return $hour;
  }
  
  /**
   * Find if a uid is in a group
   * return boolean 
   */
  function simple_LMS_user_get_group($uid) {
    //echo $uid;
    // construct my group call
    $sql = "SELECT nid FROM {og_uid} WHERE uid = %d";
    $my_group = db_query($sql, $uid);
    $obj = db_fetch_object($my_group);
          
    // if we are in a group
    return $obj->nid;
  }
  
  /**
   * Get the product title of SCORM content,
   * the nid can be get from sco_content table.
   */
  function simple_LMS_Get_scocontent_product_title($nid) {
    $node = node_load($nid);
    return $node->title;
  }

  /**
   * Check if the user has accessed at least once.
   * if not, then there is not necessary to show the view report.
   */
  function simple_LMS_Tried_scocontent($uid, $scoid, $sid) {
    $sql = "SELECT * FROM {" . SCOLearner_TABLE . "} WHERE scoid=%d AND uid=%d AND sid=%d";
    
    $r = db_query($sql, $scoid, $uid, $sid);
    
    if ($r) {
      $ary = db_fetch_array($r);
      
      if ($ary[CMI_CORE_LESSON_STATUS_DB] != CMI_INITIAL_LESSON_STATUS) {
        return TRUE;
      }
      
      return FALSE;
    }
    return FALSE;
  }

  /**
   *  Check if this user has access to a specific group based on membership
   * logic - check the user group and compare to the group to check against
   * $uid - the user to check
   * $nid - the group to check
   */
  function simple_LMS_Is_org_admin_for_group($uid, $nid) {
    ///////////TODO PLEASE FIX ME FOR ORG ADMIN
    // the group the user is in
    $user_group = simple_LMS_user_get_group($uid);
    
    // get the theme of that group
    $group_theme = simple_LMS_org_group_get_theme($nid);
    // get the theme of my group
    $my_theme = simple_LMS_org_group_get_theme($user_group);
    // match my theme to the group theme
    if ( $group_theme == $my_theme) {
      return TRUE;
    }
    //echo $user_group;
    //echo $nid;
    return FALSE;
  }
  
  function simple_LMS_org_group_get_theme($nid) {
    // need to get the theme of this group
    $sql = "SELECT og_theme FROM {og} WHERE nid = '%d'";
    $query = db_query($sql, $nid);
    if ($query && $object = db_fetch_object($query)) {
      return $object->og_theme;
    }
    return FALSE;
  }

  // this function is used in the templates to replace the username with the persons name
  function simple_LMS_get_name_from_username($username) {
    $sql = "SELECT first_name, last_name FROM {uc_addresses} WHERE uid IN ( SELECT uid FROM {users} WHERE name = '%s')";
    
    $result = db_query($sql, $username);
    
    if ($result) {
      $name_seperated = db_fetch_object($result);
      $name = $name_seperated->last_name . ", " . $name_seperated->first_name;
      return $name;
    }
    
    return "";
  }
  
  // this function will give you back the best scoreing entry from the selected user and course
  function simple_LMS_get_sid_best_score($member_id, $scoid) {
    $sql = "SELECT sid, score FROM {" . SCOLearner_TABLE . "} WHERE uid=%d AND scoid=%d";
	
	$result = db_query($sql, $member_id, $scoid);
	
	// there should be at this point but just encase
	if ($result) {
       $highscore = 0;
	   $sid = -1;
	   while ($row = db_fetch_object($result)) {
         if ($highscore < $row->score){
			$highscore = $row->score;
			$sid = $row->sid;
		 }
	   }
	   return $sid;
	}
	return NULL;
  }