<?php

 /**
  * @file
  *  Coding standard in this file. 
  *  Public function:   All functions start with Capital Letter. these functions needs to be used
  *            in other files under the same module, or needs to be exposed to outside the
  *            module by .module file.
  *  Private function:  All functions start with "_", this functions is limited in this file.
  */

  include_once('global_lib.inc');

  /**
   * The functions in this files are responsible for managing the status of the sco content
   * that the user is browsing, because of the database issue, names of standard variables 
   * are shorten, only to keep the last component of the name, such as cmi.core.score.raw, 
   * in the database table, the corresponding variable is score.
   * In addition, all functions here need to query the database as less as possible.
   */

  /**
   * Corresponding with the client-side javascript LMSInitialize function.
   * because the calling function is javascript function, the return value is by printing.
   */
  function simple_LMS_LMSInitialize() {
    global $user;
    
    $scoid = $_POST['scoid'];    
    $sid = $_POST['sid'];    
    
    $returnval = "false"; 
    
    if ( simple_LMS_initialization($user->uid, $scoid, $sid) ) {
      $returnval = "true";
    }
  
    print $returnval;
  } 
  
  /** 
   * The initialization phase needs to
   * (1): check the validation of the user.
   * (2): check the start_date.
   * (3): reset the session_time.
   * (4): if it is quiz, check if it has been finished.
   * ...more...
   */
  function simple_LMS_initialization($uid, $scoid, $sid) {
    if ( simple_LMS_check_user_valid($uid, $scoid) ) {
      $scolearner_item = simple_LMS_Get_item_from_scolearner($uid, $scoid, $sid);
    
      if ( !is_null($scolearner_item) ) {      
        if ( simple_LMS_can_restart($scolearner_item) ) {
          simple_LMS_check_startdate($scolearner_item);
        
          simple_LMS_reset_session_time($scolearner_item);
        
          if ( simple_LMS_Update_scolearner($scolearner_item) ) return TRUE;
        }
      }
    }
    
    return FALSE;
  }
  
  function simple_LMS_can_restart(&$scolearner_item) {
    //check if the quiz or course is able to be restarted...
    //for now, leave it always TRUE.    
    //check expiration date
    $d = $scolearner_item['expiration_date'];
    
    if ($d != INVALIDATE_DATE) {

      if ( strtotime($d) > time() )  return TRUE;
      
      return FALSE;
    }
  
    return TRUE;
  }
  
  function simple_LMS_reset_session_time(&$scolearner_item) {
    $scolearner_item[CMI_CORE_SESSION_TIME_DB] = CMI_INITIAL_TIME;
  }
  
  function simple_LMS_check_startdate(&$scolearner_item) {
    if ( $scolearner_item['start_date'] == INVALIDATE_DATE ) {
        
        $scolearner_item['start_date'] = simple_LMS_get_currenttime();
      
    }  
  }
  
  /**
   * check if the user is anonymous user.
   */
  function simple_LMS_check_user_valid($uid, $scoid) {
  
    //anonymous user in drupal always has the 0 uid number.
    if ($uid == 0)  return FALSE;
    
    return TRUE;
  }
  
  
  /**
   * Corresponding with the client-side javascript LMSGetValue function.
   * The request variable name is stored in $_RESQUEST['variableName']. 
   * 
   * cmi.core._children return list only lists all of the mandatory elements.
   * "- student_id - student_name - lesson_location - credit - lesson_status
   * - entry - score - total_time - exit - session_time"
   *
   * cmi.interactions._children return answers for all questions
   * the contents need to be stored in database are
   * "- id - time - student_response - result - latency".
   * 
   * cmi.core.score.raw is actually one of a group of 3 elements that 
   * relate to the course/quiz score. Respectively, there are
   * cmi.core.score.min;
   * cmi.core.score.max;
   * cmi.core.score.raw;
   * but only cmi.core.score.raw is mandatory.
   * and this is the way that LMS tells the course that it only supports
   * the cmi.core.score.raw element.
   */
  function simple_LMS_LMSGetValue() {
    global $user;
    
    $scoid = $_POST['scoid'];
	
	$sid = $_POST['sid'];
    
    $variablevalue = "";
  
    $variablename = trim($_REQUEST['variableName']);
    
    switch ($variablename) {
      case "cmi.interactions._children":
        $variablevalue = "id,time,student_response,result,latency";
        break;  
        
      case "cmi.core._children":
        $variablevalue = "student_id,student_name,lesson_location,credit,lesson_status,entry,score,total_time,exit,session_time";      
        break;
        
      case "cmi.core.score._children":
        $variablevalue = "raw";
        break;
      
      case "cmi.core.student_name":
        $variablevalue = simple_LMS_get_studentname();
        break;
        
      case "cmi.core.student_id":
        $variablevalue = simple_LMS_get_studentid();
        break;
      
      default:
        $variablevalue = simple_LMS_get_standard_scorm_variable_value($user->uid, $scoid, $sid, $variablename);
        break;
    }
    
    print $variablevalue;
  }
  
  function simple_LMS_get_standard_scorm_variable_value($uid, $scoid, $sid, $variablename) {
    if ( ($db_var = simple_LMS_Mappingname($variablename)) != NULL ) {
    
      $scolearner_item = simple_LMS_Get_item_from_scolearner($uid, $scoid, $sid);
      
      if ( !is_null($scolearner_item) ) {
      
        return $scolearner_item[$db_var];
      }
    }
    
    return "";
  }

  /**
   * This function is used to get the id of current user.
   * and is potential exposed to all of the module.
   * Real result of this function will be decided in later development schedule, 
   * with the development of credit card user information and register information.
   */
  function simple_LMS_get_studentid() {
    global $user;
    
    return $user->uid;
  }

  /**
   * This function is used to get the name of current user.
   * and is potential exposed to all of the module.
   * Real result of this function will be decided in later development schedule, 
   * with the development of credit card user information and register information.
   */
  function simple_LMS_get_studentname() {
    global $user;

    return $user->name;
  }

  /**
   * all communication should be cached on client-side.
   * so this function should not be used any more
   */
  function simple_LMS_LMSSetValue() {
    //do nothing
  }

  function simple_LMS_LMSCommit() {
    global $user;

    $scoid = $_POST['scoid'];
	$sid = $_POST['sid'];

    if ( !is_null($scolearner_item = simple_LMS_Get_item_from_scolearner($user->uid, $scoid, $sid) ) ) {
      if ( $data = $_POST['data'] ) {
        if ( !is_array($data) )  $data = array($data);
        
        $interary = array();
      
        foreach ( $data as $varname => $varvalue ) {
          if ( !is_null( $db_varname = simple_LMS_Mappingname($varname) ) ) {
            $scolearner_item[$db_varname] = $varvalue;
          }
          else {
            simple_LMS_Filter_cmi_interactions($varname, $varvalue, $interary);
          }  
        }
        
        simple_LMS_Update_scolearner($scolearner_item);
        
        simple_LMS_Update_scolearner_interaction($interary, $scolearner_item['sid']);
      }
    }
  }

  function simple_LMS_LMSFinish() {
  
    global $user;
    
    $scoid = $_POST['scoid'];
	$scoid = $_POST['sid'];
    
    if ( !is_null($scolearner_item = simple_LMS_Get_item_from_scolearner($user->uid, $scoid, $sid) ) ) {
      simple_LMS_modify_finish_credit($scolearner_item);
      
      simple_LMS_modify_finish_entry($scolearner_item);
      
      simple_LMS_modify_finish_totalTime($scolearner_item);
            
      simple_LMS_modify_finish_lessonStatus($scolearner_item);
      
      simple_LMS_modify_finish_date($scolearner_item);
      
      /**
       * ...... more modification at the finish step ......
       */
      simple_LMS_Update_scolearner($scolearner_item);
      
    }
  }
  
  function simple_LMS_modify_finish_date(&$scolearner_item) {
    $scolearner_item["finish_date"] = simple_LMS_get_currenttime();
  }
  
  function simple_LMS_modify_finish_credit(&$scolearner_item) {
    //do nothing right now;
  }
  
  function simple_LMS_modify_finish_entry(&$scolearner_item) {
    $s = trim( $scolearner_item[CMI_CORE_EXIT_DB] );
    
    switch ($s) {    
      case "suspend":
        $scolearner_item[CMI_CORE_ENTRY_DB] = "resume";
        return;
      
      //case "logout":
      //case "time-out":
      default:
        $scolearner_item[CMI_CORE_ENTRY_DB] = "";
        return;
    }
  }
  
  function simple_LMS_modify_finish_lessonStatus(&$scolearner_item) {
    $sco = simple_LMS_Get_sco_contents_object( $scolearner_item['scoid'] );
    
    if ( !is_null($sco) ) {
    
      if ( !is_null($sco->masteryscore) 
      &&   drupal_strlen( $sco->masteryscore ) > 0 
      &&  !is_null($scolearner_item[CMI_CORE_SCORE_RAW_DB]) 
      &&  drupal_strlen($scolearner_item[CMI_CORE_SCORE_RAW_DB]) ) {
        $pass_score = floatval($sco->masteryscore);
        
        $cur_score = floatval($scolearner_item[CMI_CORE_SCORE_RAW_DB]);
        
        if ($cur_score >= $pass_score) {
          $scolearner_item[CMI_CORE_LESSON_STATUS_DB] = "passed";
        }
        else {
          $scolearner_item[CMI_CORE_LESSON_STATUS_DB] = "failed";
        }
      }
    
    }
  }
  
  function simple_LMS_modify_finish_totalTime(&$scolearner_item) {
    $t = simple_LMS_calculate_seconds($scolearner_item[CMI_CORE_TOTAL_TIME_DB]);
    
    $s = simple_LMS_calculate_seconds($scolearner_item[CMI_CORE_SESSION_TIME_DB]);
    
    $scolearner_item[CMI_CORE_TOTAL_TIME_DB] = simple_LMS_calculate_scorm_time( ($t + $s) );
  }
  
  
  function simple_LMS_calculate_scorm_time($totalseconds) {
    $totalhours = intval($totalseconds/3600);
    
    $totalseconds -= $totalhours * 3600;
    
    $totalminutes = intval($totalseconds/60);
    
    $totalseconds -= $totalminutes * 60;
    
    // reformat to comply with the SCORM data model
    $totaltime = sprintf("%04d:%02d:%02d", $totalhours, $totalminutes, $totalseconds);
    
    return $totaltime;
  }
  
  function simple_LMS_LMSGetLastError() {
    
    //ignore currently
  }
  
  function simple_LMS_LMSGetDiagnostic() {
  
    //ignore currently
  }
  
  function simple_LMS_LMSGetErrorString() {
    
    //ignore currently
  }