<?php

/**
 * @file
 * Coding standard in this file. 
 * Public function:   All functions start with Capital Letter. these functions needs to be used
 *            in other files under the same module, or needs to be exposed to outside the
 *            module by .module file.
 *  Private function:  All functions start with "_", this functions is limited in this file.
 *  Database Table  :   All table names need to be defined as static variable defined by using php "define"
 */

include_once('global_lib.inc');

/**
 * list all of the sco content under specific directory.
 * if the content is already in the database, the checkbox before it will be checked
 * otherwise, the checkbox is unchecked.
 */
function simple_LMS_Add_sco_content_form() {  
  $sco_list_underdir = simple_LMS_readdir_underdir(sco_url_basedir);

  $sco_list_underdb  = simple_LMS_readsco_fromdb();
  
  $form = array();
  
  $form['sco'] = array(
    '#type'     => 'fieldset',
    '#title'    => t('Sharable Content Objects'),
    '#collapsible'  => TRUE,
    '#collapsed'  => FALSE,
    '#tree'      => TRUE,
  );
  
  foreach ($sco_list_underdir as $dir) {
    
      $check = isset( $sco_list_underdb[$dir] ) ? 1 : 0;
      
      $appendix = !$check ? "<font color = '#ff0000' style = 'display:inline;'> (new added contents)</font>" : "";
      
      $form['sco'][$dir] = array(
        '#type' => 'checkbox',
        '#title' => $dir . $appendix,
        '#default_value' => $check,
        '#return_value' => 1, 
      );
    
  }   
  
  $form['sco']['submit'] = array(
    '#type' => 'submit',
    '#title' => t('Save Configuration'),
    '#value' => t('Save Configuration'),
  );
  
  return $form;
}

/**
 *  whenever add_sco_content_form is submitted, this function is invoked.
 */
function simple_LMS_Add_sco_content_form_submit($form, &$form_state) {
  global $user;

  $sco_list_underdir = simple_LMS_readdir_underdir(sco_url_basedir);
  
  $sco_list_underdb  = simple_LMS_readsco_fromdb();
  
  foreach ($sco_list_underdir as $dir) {
    
    if ( isset( $form_state['values']['sco'][$dir] ) ) {
      if ( $form_state['values']['sco'][$dir] == 1 && !isset($sco_list_underdb[$dir])) {
        simple_LMS_Add_new_scocontent($dir);
      }
    }
  }
}

/**
 *  This function list all of the sco contents a user has.
 */
function simple_LMS_Show_User_contents() {
  //right now, I suppose the user is current user.
  global $user;

  $sco_ary = simple_LMS_Get_user_contents($user->uid);
  
  $titles = array( t("Name"), 
      t("Expiration Date"),
      t("Take Course"),
      t("View Report") );
  
  $rows = array();
  
  foreach ($sco_ary as $sco) {
  
    $rows[] = array(t(simple_LMS_Get_scocontent_product_title($sco['product_nid'])),
            t($sco['expirationdate']),
            simple_LMS_Is_expiration_date_past($sco['expirationdate']) ? "Expired" : l(t('Take Course'), 'lms/see_sco_contents/' . $sco['scoid'] . "/" . $sco['sid']),
            !simple_LMS_Tried_scocontent($user->uid, $sco['scoid'], $sco['sid']) ? "Not Taken Yet" : l(t('View report'), 'lms/see_my_report/' . $sco['scoid'] . "/" . $sco['sid']), );

  }
  
  return theme('table', $titles, $rows);
}

/**
 * This function is to create the entry from which user can view the concrete sco content
 * The scoid is passed by url, check the .module file menu:$items['lms/see_sco_contents/%']
 */
function simple_LMS_Present_sco_content() {
  global $user;
  
  $scoid = arg(2);
  
  $sid = arg(3);
  
  $sco = simple_LMS_Get_sco_contents_object($scoid);
  
  if ( !is_null( $sco ) ) {
  
    $sco_links = $sco->scolinks;
    
    $curpath = drupal_get_path('module', MODULE_NAME);
    
    $api_file = '/'. $curpath . sco_api_file . "?id=" . $scoid . "&sid=" . $sid;

    simple_LMS_get_sco_entry($api_file, $sco_links);
  }
  
}

/**
 * This function is used created a html file by bypassing the theme.
 */
function simple_LMS_get_sco_entry($api_file, $sco_links) {
  print "<frameset frameborder=\"0\" framespacing=\"0\" border=\"0\" rows=\"0,*\" cols=\"*\" >";
  print "<frame src=\"" . $api_file ."\" name=\"API\" noresize>";
  print "<frame src=\"" . $sco_links . "\" name=\"course\">";
  print "</frameset>";
}

/**
 * read all of the content stored in database.
 */
function simple_LMS_readsco_fromdb() {
  $sco_list_underdb = array();
  
  $sql_statement = "SELECT scoid, sconame FROM {". SCO_TABLE ."}";
  
  $result = db_query($sql_statement);
  
  while ( $sco = db_fetch_object( $result ) ) {
  
    $sco_list_underdb[$sco->sconame] = array($sco->scoid, $sco->sconame);

  }
  
  return $sco_list_underdb;
}

/**
 *  read all directories under specific directories.
 */
function simple_LMS_readdir_underdir($basedir) {
  $dir_list_underdir = array();
  
  $dir_handle = opendir($basedir) or die("Unable to open $basedir");
  
  //open the base directory which includes all of the sco contents(course,quiz)
  while ( FALSE !== ( $file = readdir( $dir_handle ) ) ) {
  
    $dir = $basedir . '/' . $file;
    
    if (is_dir($dir) && $file != '.' && $file != '..') {
      
        $dir_list_underdir[] = $file;
    }
  }
    
  closedir($dir_handle);
  
  return $dir_list_underdir;
}


/**
 * This function read the data from Session[GROUP_STATISTIC_EXCEL]
 * to format a Excel file.
 * I Copy this code from http://www.the-art-of-web.com/php/dataexport/#section_1
 */
function simple_LMS_down_load_group_statistic_Excel() {
  $data = $_SESSION[GROUP_STATISTIC_EXCEL];

  $filename = "". $data['course_name'] . ".xls"; 

  header("Content-Disposition: attachment; filename=\"$filename\""); 
  header("Content-Type: application/vnd.ms-excel"); 
  
  $firstrow = FALSE;
  $flag = FALSE; 
  foreach ( $data as $row ) {
    if ( !$firstrow ) {  
      $firstrow = TRUE;
      continue;
    }
    if ( !$flag ) { 
      # display field/column names as first row 
      echo implode("\t", array_keys($row)) . "\n"; 
      $flag = TRUE; 
    } 
    array_walk($row, 'simple_LMS_cleanData'); 
    echo implode("\t", array_values($row)) . "\n"; 
  }
  exit;
}

/**
 * Comment ME
 */
function simple_LMS_cleanData(&$str) {
  $str = preg_replace("/\t/", "\\t", $str); 
  $str = preg_replace("/\r?\n/", "\\n", $str);  
  if ( strstr( $str, '"' ) ) 
    $str = '"' . str_replace('"', '""', $str) . '"';  
} 

/**
 * Present all user in my group by using drupal table
 */
function simple_LMS_List_users_in_my_group() {
  global $user;
  
  $nid = simple_LMS_user_get_group($user->uid);
      
  $useraryary = simple_LMS_Get_users_in_group($nid);
  
  if ( $useraryary ) {
  
    $titles = array(
            t("Group Name"), 
            t("Member Name"),
            t("Member Email"),
            t("View Report"), 
            );
    
    $rows = array();
    
    foreach ( $useraryary as $group_id => $userary ) {
      
      $count = 0;
      
      $group_description = "";
      
      foreach ( $userary as $member ) {
        
        if ( $count == 0 ) {
        
          $count++;
          
          //this first element of this array is the group description
          $group_description = $member; 
        
          continue;
        }
        
        $rows[] = array(
                t($group_description),
                t($member->name),
                t($member->mail),
                l( t('View report'), 'lms/see_member_report/' . $member->uid . '/-1'), );
      }
      
      $rows[] = array(
      l(t('Group Statistic Information of ' . $group_description), 'lms/see_group_statistic/' . $group_id . '/-1'));
    }
    
    $output = theme('table', $titles, $rows);
    //$output .= theme('pager', NULL, 2, 0);

    return $output;
  }
  
  return NULL;
}

/**
 * see someone's own quiz or course report
 */
function simple_LMS_See_my_report() {
  global $user;
  
  $scoid = arg(2);
  $sid = arg(3);
  
  $sco_content = simple_LMS_Get_sco_contents_object($scoid);
  
  $sco_main_report = simple_LMS_Get_item_from_scolearner($user->uid, $scoid, $sid, TRUE);
  
  $sco_questions_report_list = simple_LMS_Get_question_answers($sid);

  $member_obj = simple_LMS_Get_user_info($user->uid);
  
  return theme('individual_report', $sco_content, $sco_main_report, $sco_questions_report_list, NULL, $member_obj);
}

/**
 * list all of the reports of the specific member of the group.
 */
function simple_LMS_See_member_report() {
  //user id of this member.
  $member_id = arg(2);
  
  //sco id
  $scoid = arg(3);
  
  // this is the potential downfall of using a tri key access point we choose to use the best score method
  $sid = simple_LMS_get_sid_best_score($member_id, $scoid);
  
  $member_obj = simple_LMS_Get_user_info($member_id);
  
  $sco_list = simple_LMS_Get_user_contents($member_id);
  
  if ( $scoid == -1 ) {
    return theme('individual_report', NULL, NULL, NULL, $sco_list, $member_obj);
  }
  else {
    $sco_content = simple_LMS_Get_sco_contents_object($scoid);
    
    $sco_main_report = simple_LMS_Get_item_from_scolearner($member_id, $scoid, $sid, TRUE);
    
    $sco_questions_report_list = simple_LMS_Get_question_answers($sid);
    
    return theme('individual_report', $sco_content, $sco_main_report, $sco_questions_report_list, $sco_list, $member_obj);
  }
}

/**
 * List all of the statistic information of the group users in graph chart format.
 */
function simple_LMS_See_group_statistic_info() {
  global $user;

  $group_id = arg(2);

  $scoid = arg(3);

  // check permisions here in order (site admin, group admin, org admin)
  if ( simple_LMS_Is_admin_of_website_byUserObj($user) || 
  simple_LMS_Is_admin_of_group($user->uid, $group_id) || 
  simple_LMS_Is_org_admin_for_group($user->uid, $group_id) ||
  $user->uid == uc_np_training_works_purchase_managed_group_get($group_id,'man')) {
    /*  $sco_content;
      $sco_main_report;
      $sco_list;
      $members_info;
      $group_id;*/

    if ($scoid == -1) {

      $sco_list = simple_LMS_Get_sco_contents_of_group($group_id);

      return theme('group_report', NULL, NULL, $sco_list, NULL, $group_id);
    }
    else {
      $sco_content = simple_LMS_Get_sco_contents_object($scoid);

      $sco_main_report = simple_LMS_Get_scolearner_items_for_group($group_id, $scoid);

      $sco_list = simple_LMS_Get_sco_contents_of_group($group_id);

      $members = simple_LMS_Get_users_in_group($group_id);

      $members_info = $members[$group_id];

      return theme('group_report', $sco_content, $sco_main_report, $sco_list, $members_info, $group_id);
    }
  }
}
/** 
 * Get the group information
 * Return assco array of info
 * array contains theme, group_name, group_admin, admin_email, description 
 * add more functionality if needed
 */
function simple_LMS_get_group_data($gid, $org) {
  // super long sql statement so build it up piece by piece
  $sql_select = "SELECT node.title AS group_name, og.og_theme, og.og_description, users.uid, og.og_directory AS is_org, users.name AS user_name, users.mail";
  $sql_tables = "FROM {node , og, og_uid, users}";
  $sql_where = "WHERE node.nid = og.nid AND og.nid = og_uid.nid AND og_uid.uid = users.uid AND og_uid.is_admin = 1 AND og.nid = %d";
  
  //build it all together now
  $sql = $sql_select . " " . $sql_tables . " " . $sql_where;
  
  // perform the db functions
  $result = db_query($sql, $gid);
  
  // for each group by user (technically a group by group admin sort)
  while ( $group = db_fetch_object($result) ) {
    // Whether you want orgs or non orgs
    if ($group->is_org == $org) {
      // since we are grabing out the group with group admins associatted with them we need to eliminate site admins
      if ( !simple_LMS_Is_admin_of_website_byUserId($group->uid)) {
        // filter out data that is wanted
        $data = array('name' => $group->group_name, 'admin_user' => $group->user_name, 'email' => $group->mail, 'og_theme' => $group->og_theme, 'discription' => $group->og_description);
        return $data;
      }
    }
  }
  return NULL;
}

// returns the theme and nid of all orgs in the og table
function simple_LMS_get_orgs() {
  $sql = "SELECT nid FROM {content_type_group} WHERE field_parent_org_value = 'Org'";
  
  $query = db_query($sql);
  
  if ($query) {
    $all_orgs = array();

    while ($an_org = db_fetch_object($query)) {
      $all_orgs[] = array('nid' => $an_org->nid);
    }

    return $all_orgs;
  }

  return NULL;
}

/**
 * View the chart for a specific course/quiz which belong to a group
 * index 2: group id
 * index 3: scoid
 */
function simple_LMS_View_group_statistic_chart() {
  global $user;

  $group_id = arg(2);

  $scoid = arg(3);

  if ( simple_LMS_Is_admin_of_group($user->uid, $group_id) || 
  simple_LMS_Is_admin_of_website_byUserObj($user) ||
  $user->uid == uc_np_training_works_purchase_managed_group_get($group_id,'man')) {
  
    $sco_list = simple_LMS_Get_sco_contents_of_group($group_id);

    if ( $scoid == -1) {
      $content = theme('chart_report', 
          NULL, 
          NULL, 
          NULL, 
          $sco_list, 
          $group_id);
      
      return $content;
    }
    else {                
      $sco_questions_list = simple_LMS_Get_question_answers_forgroup($group_id, $scoid);
              
      $sco_content = simple_LMS_Get_sco_contents_object($scoid);
              
      $sco_main_report = simple_LMS_Get_scolearner_items_for_group($group_id, $scoid);
              
      $members = simple_LMS_Get_users_in_group($group_id);
              
      $members_info = $members[$group_id];              
              
      $content = theme('chart_report', 
          $sco_main_report, 
          $sco_content, 
          $sco_questions_list, 
          $sco_list, 
          $group_id);

      return $content;
    }
  }
}

/**
 * Group user block ...
 */
function simple_LMS_View_group_info() {
  // get the viewer of this page account info
  global $user;

  // passed in argument from path
  $nid = arg(2);

  // does this user have access to this group?
  // one of three ways (site admin, group admin, org admin)
  if ( simple_LMS_Is_admin_of_website_byUserObj($user) || 
  simple_LMS_Is_admin_of_group($user->uid, $nid) || 
  simple_LMS_Is_org_admin_for_group($user->uid, $nid) ||
  $user->uid == uc_np_training_works_purchase_managed_group_get($nid,'man')) {
    /**
     * Group information
     */

    // construct the headers of the tables
    $header = array(
      array('data' => t('User Name'), 'field' => 'uid', 'sort' => 'asc'),
      array('data' => t('User Email')),
      array('data' => t('Report Link'))
    );

    // this will soon be taken out because the top level is orgs so should show that first
    /*
    // if we are the admin of the website then we want all groups
    if (Is_admin_of_website_byUserObj($user) && $nid = -1) {
      // default it
      $output = "";
      
      // all groups unique
      $all_groups = db_query("SELECT DISTINCT nid FROM {og_uid}");
      
      while($group_nid = db_fetch_object($all_groups)) {
        // load the group before hand for org testing 
        $group_data = simple_LMS_get_group_data($group_nid->nid,0);

        $sql = "SELECT * FROM {og_uid} WHERE nid = $group_nid->nid";
        
        // allows us to sort the table
        $sql .= tablesort_sql($header);
        // get the current page of results
        $result = pager_query($sql, 50, 0 , NULL, $keys);

        // start constructing the individual rows
        $rows = array();
        
        while ($data = db_fetch_object($result)) {
          // remember no showing the site admins
          if (!(Is_admin_of_website_byUserId($data->uid))) {
            if ( $data->uid == $user->uid) {
              // if you want to see admin as part table 
              //$row = array(check_plain('YOU'), 'admin', '');
              //$rows[] = $row;
            }
            else {
              $a_user = user_load($data->uid); 
              $a_username = $a_user->name;
              $a_email = $a_user->mail;
              
              /// WHY ARE ORGS STILL SHOWING UP
              if ($a_username == "Bitch")
              {
                echo "problem";
              }
              $row = array(check_plain($a_username),check_plain($a_email), l(t('Report'),'lms/see_member_report/' . $data->uid . '/-1'));
              $rows[] = $row;
            }
          }
        }
        
        $rows[] = array(l(t('View Group Statistic Information'), 'lms/see_group_statistic/'.$group_nid->nid . '/-1'));
        

        // deal with an empty set
        if (empty($rows)) {
          $empty_message = $keys ? t('No Users Found In this Group.') : t('No Users found.') ;
          $rows[] = array(array('data' => $empty_message));
        }

        
        $group_name = $group_data['name'];
        $output .= "<h2>Group Reports For $group_name</h2>";
        
        $group_admin = $group_data['admin_user'];
        $admin_email = $group_data['email'];
        
        $output .= "<h3>Group Admin: $group_admin, Admin Email: $admin_email</h3>";
        
        // display the information
        $output .= theme('table', $header, $rows);
        $output .= theme('pager', NULL, 50, 0);
      }

      // let drupal handle print and echo
      return $output;
    }*/
    // non admin of the site regular user
    //else
    // sql statement for gathering the groups
    $sql = "SELECT * FROM {og_uid} WHERE nid = '%d'";
    
    $keys = array($nid);
    // allows us to sort the table
    $sql .= tablesort_sql($header);
    // get the current page of results
    $result = pager_query($sql, 50, 0 , NULL, $keys);

    // start constructing the individual rows
    $rows = array();
    //echo "my id = $user->uid";
    while ($data = db_fetch_object($result)) {
      // remember no showing the site admins
      if ( ! simple_LMS_Is_admin_of_website_byUserId($data->uid) ) {
        // our uid matched the admin of the group
        if ( $data->uid == $user->uid) {
          /* // if you want to see admin as part of table re add this
          $a_user = user_load($data->uid); 
          $a_username = $a_user->name;
          $a_email = $a_user->mail;
          $row = array(check_plain($a_username),check_plain($a_email), check_plain("admin"));
          $rows[] = $row;*/
        }
        else {
          $a_user = user_load($data->uid); 
          $a_username = $a_user->name;
          $a_email = $a_user->mail;
          $row = array(check_plain($a_username), check_plain($a_email), l(t('Report'), 'lms/see_member_report/' . $data->uid . '/-1'));
          $rows[] = $row;
        }
      }
    }
    
    // deal with an empty set
    if (empty($rows)) {
      $empty_message = $keys ? t('No Users Found In this Group.') : t('No Users found.') ;
      $rows[] = array(array('data' => $empty_message));
    }
    
    // the last row in the table to show group wide stats
    $rows[] = array(l(t('View Group Chart Information'), 'lms/view_group_statistic_chart/' . $nid . '/-1'), '' , l(t('View Group Statistic Information'), 'lms/see_group_statistic/' . $nid . '/-1'), );

    // get group info
    $group_data = simple_LMS_get_group_data($nid, 0);
    $group_name = $group_data['name'];
    
    if (!$group_name) {
      return "<p>no access, or problem has occured</p>";
    }

    // title
    $output = "<div id='simple-lms-group-page'> <h2>Group Reports For $group_name</h2>";
    
    // get admin info
    $group_admin = $group_data['admin_user'];
    $admin_email = $group_data['email'];
    
    // admin section
    $output .= "<h3>Group Admin: $group_admin, Admin Email: $admin_email</h3>";
    
    // display the information
    $output .= theme('table', $header, $rows);
    $output .= theme('pager', NULL, 50, 0);
	$output .= "</div>";

    // let drupal handle print and echo
    return $output;
  }
}

/**
 * From lms/view_organization_group/%
 */
function simple_LMS_View_organization_group() {
  $group_id = arg(2);
  
  // grab the account viewing this content
  global $user;
  
  // construct the headers of the table
  $header = array(
    // there is currently no way to sort this table
    //array('data' => t('Group Name'), 'field' => 'theme', 'sort' => 'asc'),
    array('data' => t('Group Name')),
    array('data' => t('Members')),
    array('data' => t('Statistic')),
    array('data' => t('Chart')),
  );
  
  // if we are the admin of the website then we want all orgs
  if (simple_LMS_Is_admin_of_website_byUserObj($user) && $group_id == -1) {
    // get all gids of all orgs
    $all_orgs = simple_LMS_get_orgs();
    
    // default output
    $output = "<div id='simple-lms-admin-page'>"; 
    $output .= "<h2>Group Reports</h2>";
	$output .= "<p>Linked below are Statistical and Chart reports for your Group.  Click on the links to view aggregate or individual statistical and chart reports.</p>";
    
    // for all of them display thier charts
    foreach ($all_orgs as $an_org) {
      $org_id = $an_org['nid'];
      // org table is based on theme
      $sql = "SELECT * FROM {og} WHERE og_theme IN (SELECT og_theme FROM {og} WHERE nid = %d)";
      
      $keys = array($org_id);
      
      // allows us to sort the table (uncomment when sorting is fixed
      //$sql .= tablesort_sql($header);
      
      // get the current page of results
      $result = pager_query($sql, 50, 0 , NULL, $keys);
	  
	  $theme = "";
      
      // start constructing the individual rows
      $rows = array();
      while ($data = db_fetch_object($result)) {
        if ( $data->og_directory == 1 )
		{
	      $theme = $data->og_theme;
		}
		else {
          $gid = $data->nid;
          $group_data = simple_LMS_get_group_data($gid, 0);
          $group_name = $group_data['name'];
		  if (!$group_name) {
		    $group_node = node_load($gid);
		    $group_name = $group_node->title;
			$error = "No users found";
            $row = array(check_plain($group_name),check_plain($error),NULL,NULL);			
		  } 
		  else {
            $row = array(check_plain($group_name ), l(t('Members'), "lms/view_group_info/$data->nid"), l(t('Statistic'), "lms/see_group_statistic/$data->nid/-1"), l(t('Chart'), "lms/view_group_statistic_chart/$gid/-1"));
		  }
          $rows[] = $row;
		}
      }

      // deal with an empty set
      if (empty($rows)) {
        $empty_message = $keys ? t("No Groups Found for $theme.") : t('No Groups found.') ;
        $rows[] = array(array('data' => $empty_message));
      }

      
      $output .= "<h3>$theme</h3>";
      // display the information
      $output .= theme('table', $header, $rows);
      $output .= theme('pager', NULL, 50, 0);
	  
    }
	// this will be where loosely hanging groups go same structure as above not based on theme
      
    // loose table is based on nids
    $sql = "SELECT * FROM {og} WHERE nid IN (SELECT nid FROM {content_type_group} WHERE field_parent_org_value = 'None')";
      
    // allows us to sort the table (uncomment when sorting is fixed
    //$sql .= tablesort_sql($header);
      
    // get the current page of results
    $result = pager_query($sql, 50, 0 , NULL, NULL);
	  
	$theme = "";
      
    // start constructing the individual rows
    $rows = array();
    while ($data = db_fetch_object($result)) {
        $gid = $data->nid;
        $group_data = simple_LMS_get_group_data($gid, 0);
        $group_name = $group_data['name'];
		if (!$group_name) {
		  $group_node = node_load($gid);
		  $group_name = $group_node->title;
	      $error = "No users found";
          $row = array(check_plain($group_name),check_plain($error),NULL,NULL);			
		} 
		else {
          $row = array(check_plain($group_name ), l(t('Members'), "lms/view_group_info/$data->nid"), l(t('Statistic'), "lms/see_group_statistic/$data->nid/-1"), l(t('Chart'), "lms/view_group_statistic_chart/$gid/-1"));
		}
        $rows[] = $row;
    }

    // deal with an empty set
    if (empty($rows)) {
      $empty_message = $keys ? t("No Groups Found for $group_name.") : t('No Groups found.') ;
      $rows[] = array(array('data' => $empty_message));
    }

      
    $output .= "<h3>Others</h3>";
    // display the information
    $output .= theme('table', $header, $rows);
    $output .= theme('pager', NULL, 50, 0);
	
	$output .= "</div>";
    // let drupal handle print and echo
    return $output;
  }
  // regular org admin
  // test for this by if this person belongs to the org
  elseif ( $group_id != -1 && simple_LMS_Is_org_admin_for_group($user->uid, $group_id)) {
    $theme = simple_LMS_org_group_get_theme($group_id);
    
    // org table is based on theme
    $sql = "SELECT * FROM {og} WHERE og_theme = '%s' AND og_directory = 0";
    
    // keys
    $keys = array($theme);
    
    // get the current page of results
    $result = pager_query($sql, 50, 0 , NULL, $keys);
    
    // allows us to sort the table
    $sql .= tablesort_sql($header);
    
    // start constructing the individual rows
    $rows = array();
    while ($data = db_fetch_object($result)) {
      $gid = $data->nid;
      $group_data = simple_LMS_get_group_data($gid, 0);
      $group_name = $group_data['name'];
      if (!$group_name) {
		  $group_node = node_load($gid);
		  $group_name = $group_node->title;
	      $error = "No users found";
          $row = array(check_plain($group_name),check_plain($error),NULL,NULL);			
		} 
		else {
          $row = array(check_plain($group_name ), l(t('Members'), "lms/view_group_info/$data->nid"), l(t('Statistic'), "lms/see_group_statistic/$data->nid/-1"), l(t('Chart'), "lms/view_group_statistic_chart/$gid/-1"));
		}
      $rows[] = $row;
    }

    // deal with an empty set
    if (empty($rows)) {
      $empty_message = $keys ? t("No Groups Found for $theme.") : t('No Groups found.') ;
      $rows[] = array(array('data' => $empty_message));
    }

    $output = "<div id='simple-lms-admin-page'><h2>Group Reports</h2>";
	$output .= "<p>Linked below are Statistical and Chart reports for your Group.  Click on the links to view aggregate or individual statistical and chart reports.</p>";
    $output .= "<h3>$theme</h3>";
    // display the information
    $output .= theme('table', $header, $rows);
    $output .= theme('pager', NULL, 50, 0);
	$output .= "</div>";
    // let drupal handle print and echo
    return $output;
  }
}

function simple_LMS_view_report_stat_link() {
  // get the user object 
  global $user;
  // default the output
  $output = "";
  
  // the group of this user
  $group_id = simple_LMS_user_get_group($user->uid);
  
  if ( simple_LMS_Is_admin_of_website_byUserObj($user) ) {
    $output .= "<p class='gotolink'><a href='/lms/view_organization_group/-1'>Group Reports</a></p>";
  }
  if ( simple_LMS_Is_admin_by_role($user, GROUP_ADMIN) ) {
    // get the group this person is in
    $group_id = simple_LMS_user_get_group($user->uid);
    $output .= "<p class='gotolink'><a href='/lms/view_group_info/$group_id'>Group Reports</a></p>";
  }
  if ( simple_LMS_Is_admin_by_role($user, ORGANIZATION_ADMIN) ) {
    // get the org this person is in
    $group_id = simple_LMS_user_get_group($user->uid);
    $output .= "<p class='gotolink'><a href='/lms/view_organization_group/$group_id'>Organization Group Reports</a></p>";
  }
  
  return $output;
}